using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Data;
using PSLibrary = Microsoft.Office.Project.Server.Library;

namespace MicrosoftFrance.MCS.EPM2007
{
    class PSICustomFields
    {
        public static void CreateCustomField(string entity, string name, byte type)
        {
            CustomFieldsWS.CustomFieldDataSet dsCustomFields = new CustomFieldsWS.CustomFieldDataSet();
            CustomFieldsWS.CustomFieldDataSet.CustomFieldsRow cfrow = dsCustomFields.CustomFields.NewCustomFieldsRow();
            cfrow.MD_ENT_TYPE_UID = new Guid(entity);
            cfrow.MD_PROP_NAME = name;
            cfrow.MD_PROP_TYPE_ENUM = type;
            cfrow.MD_PROP_UID = Guid.NewGuid();
            dsCustomFields.CustomFields.AddCustomFieldsRow(cfrow);
            Program.customfields.CreateCustomFields(dsCustomFields, false, true);
        }

        public static void CreateCustomFieldProjectText(string name)
        {
            CreateCustomField(PSLibrary.EntityCollection.Entities.ProjectEntity.UniqueId,
                name, (byte)PSLibrary.CustomField.Type.TEXT);
        }

        public static void CreateCustomFieldTaskText(string name)
        {
            CreateCustomField(PSLibrary.EntityCollection.Entities.TaskEntity.UniqueId,
                name, (byte)PSLibrary.CustomField.Type.TEXT);
        }

        public static void CreateCustomFieldTaskNum(string name)
        {
            CreateCustomField(PSLibrary.EntityCollection.Entities.TaskEntity.UniqueId,
                name, (byte)PSLibrary.CustomField.Type.NUMBER);
        }

        public static void CreateCustomFieldResourceText(string name)
        {
            CreateCustomField(PSLibrary.EntityCollection.Entities.ResourceEntity.UniqueId,
                name, (byte)PSLibrary.CustomField.Type.TEXT);
        }

        public static Guid GetCustomFieldUID(string cfName)
        {
            CustomFieldsWS.CustomFieldDataSet cfDs = new CustomFieldsWS.CustomFieldDataSet();

            PSLibrary.Filter cfFilter = new PSLibrary.Filter();
            cfFilter.FilterTableName = cfDs.CustomFields.TableName;
            cfFilter.Fields.Add(new PSLibrary.Filter.Field(cfDs.CustomFields.TableName, cfDs.CustomFields.MD_PROP_UIDColumn.ColumnName, PSLibrary.Filter.SortOrderTypeEnum.None));
            cfFilter.Fields.Add(new PSLibrary.Filter.Field(cfDs.CustomFields.TableName, cfDs.CustomFields.MD_PROP_NAMEColumn.ColumnName, PSLibrary.Filter.SortOrderTypeEnum.Asc));

            PSLibrary.Filter.FieldOperator fp = new PSLibrary.Filter.FieldOperator(PSLibrary.Filter.FieldOperationType.Equal, cfDs.CustomFields.MD_PROP_NAMEColumn.ColumnName, cfName);
            cfFilter.Criteria = fp;

            cfDs = Program.customfields.ReadCustomFields(cfFilter.GetXml(), false);

            if (cfDs.CustomFields.Count == 1)
                return cfDs.CustomFields[0].MD_PROP_UID;

            return Guid.Empty;
        }

        public static Guid GetLTByName(string ltname, LookupTableWS.LookupTableDataSet ltDataSet)
        {
            Guid ltUID = Guid.Empty;
            try
            {
                ltDataSet.LookupTables.DefaultView.RowFilter = "LT_NAME = '" + ltname + "'";
                ltUID = new Guid(ltDataSet.LookupTables.DefaultView[0]["LT_UID"].ToString());
            }
            catch 
            {
                throw new Exception("La lookup table " + ltname + " n'a pas t cre.");
            }
            return ltUID;
        }

        public static Hashtable GetLTValues(string ltname, LookupTableWS.LookupTableDataSet ltDataSet)
        {
            Hashtable values = new Hashtable();
            string ltStructUID = "", ltCode = "";
            Guid ltUID = GetLTByName(ltname, ltDataSet);
            ltDataSet.LookupTableTrees.DefaultView.RowFilter = "LT_UID = '" + ltUID.ToString() + "'";
            for (int i = 0; i < ltDataSet.LookupTableTrees.DefaultView.Count; i++)
            {
                ltCode = ltDataSet.LookupTableTrees.DefaultView[i]["LT_VALUE_TEXT"].ToString();
                ltStructUID = ltDataSet.LookupTableTrees.DefaultView[i]["LT_STRUCT_UID"].ToString();
                values.Add(ltCode, ltStructUID);
            }
            return values;
        }

        public static string GetMD_PROP_UID(string name)
        {
            CustomFieldsWS.CustomFieldDataSet customFieldsDS = new CustomFieldsWS.CustomFieldDataSet();
            customFieldsDS = Program.customfields.ReadCustomFields("", false);

            customFieldsDS.CustomFields.DefaultView.RowFilter = "MD_PROP_NAME = " + "'" + name + "'";
            if (customFieldsDS.CustomFields.DefaultView.Count > 0)
            {
                return customFieldsDS.CustomFields.DefaultView[0]["MD_PROP_UID"].ToString();
            }
            else
            {
                throw new Exception("Pas de CF nomm " + name);
            }
        }

        public static string GetMD_PROP_UID_SECONDARY(string name)
        {
            CustomFieldsWS.CustomFieldDataSet customFieldsDS = new CustomFieldsWS.CustomFieldDataSet();
            customFieldsDS = Program.customfields.ReadCustomFields("", false);

            customFieldsDS.CustomFields.DefaultView.RowFilter = "MD_PROP_NAME = " + "'" + name + "'";
            if (customFieldsDS.CustomFields.DefaultView.Count > 0)
            {
                return customFieldsDS.CustomFields.DefaultView[0]["MD_PROP_UID_SECONDARY"].ToString();
            }
            else
            {
                throw new Exception("Pas de CF nomm " + name);
            }
        }

        public static void AddProjectCustomField(Guid prjID, string name, string value, ProjectWS.ProjectDataSet project)
        {
            ProjectWS.ProjectDataSet.ProjectCustomFieldsRow cfrow = project.ProjectCustomFields.NewProjectCustomFieldsRow();
            cfrow.PROJ_UID = prjID;
            cfrow.MD_PROP_UID = new Guid(GetMD_PROP_UID(name));
            cfrow.TEXT_VALUE = value;
            cfrow.CUSTOM_FIELD_UID = Guid.NewGuid();
            project.ProjectCustomFields.AddProjectCustomFieldsRow(cfrow);
        }

        public static void AddProjectCustomField(Guid prjID, string name, Guid ltcode, ProjectWS.ProjectDataSet project)
        {
            ProjectWS.ProjectDataSet.ProjectCustomFieldsRow cfrow = project.ProjectCustomFields.NewProjectCustomFieldsRow();
            cfrow.PROJ_UID = prjID;
            cfrow.MD_PROP_UID = new Guid(GetMD_PROP_UID(name));
            cfrow.CODE_VALUE = ltcode;
            cfrow.CUSTOM_FIELD_UID = Guid.NewGuid();
            project.ProjectCustomFields.AddProjectCustomFieldsRow(cfrow);
        }

        public static void AddProjectCustomField(Guid prjID, string name, decimal value, ProjectWS.ProjectDataSet project)
        {
            ProjectWS.ProjectDataSet.ProjectCustomFieldsRow cfrow = project.ProjectCustomFields.NewProjectCustomFieldsRow();
            cfrow.PROJ_UID = prjID;
            cfrow.MD_PROP_UID = new Guid(GetMD_PROP_UID(name));
            cfrow.NUM_VALUE = value;
            cfrow.CUSTOM_FIELD_UID = Guid.NewGuid();
            project.ProjectCustomFields.AddProjectCustomFieldsRow(cfrow);
        }

        public static void AddTaskCustomField(Guid prjID, Guid taskID, string name, string value, ProjectWS.ProjectDataSet project)
        {
            ProjectWS.ProjectDataSet.TaskCustomFieldsRow cfrow = project.TaskCustomFields.NewTaskCustomFieldsRow();
            cfrow.PROJ_UID = prjID;
            cfrow.TASK_UID = taskID;
            cfrow.MD_PROP_UID = new Guid(GetMD_PROP_UID(name));
            cfrow.TEXT_VALUE = value;
            cfrow.CUSTOM_FIELD_UID = Guid.NewGuid();
            project.TaskCustomFields.AddTaskCustomFieldsRow(cfrow);
        }

        public static void AddTaskCustomField(Guid prjID, Guid taskID, string name, decimal number, ProjectWS.ProjectDataSet project)
        {
            ProjectWS.ProjectDataSet.TaskCustomFieldsRow cfrow = project.TaskCustomFields.NewTaskCustomFieldsRow();
            cfrow.PROJ_UID = prjID;
            cfrow.TASK_UID = taskID;
            cfrow.MD_PROP_UID = new Guid(GetMD_PROP_UID(name));
            cfrow.NUM_VALUE = number;
            cfrow.CUSTOM_FIELD_UID = Guid.NewGuid();
            project.TaskCustomFields.AddTaskCustomFieldsRow(cfrow);
        }

        public static void AddTaskCustomField(Guid prjID, Guid taskID, string name, Guid ltcode, ProjectWS.ProjectDataSet project)
        {
            ProjectWS.ProjectDataSet.TaskCustomFieldsRow cfrow = project.TaskCustomFields.NewTaskCustomFieldsRow();
            cfrow.PROJ_UID = prjID;
            cfrow.TASK_UID = taskID;
            cfrow.MD_PROP_UID = new Guid(GetMD_PROP_UID(name));
            cfrow.CODE_VALUE = ltcode;
            cfrow.CUSTOM_FIELD_UID = Guid.NewGuid();
            project.TaskCustomFields.AddTaskCustomFieldsRow(cfrow);
        }

        public static void AddTaskCustomField(Guid prjID, Guid taskID, string name, DateTime datevalue, ProjectWS.ProjectDataSet project)
        {
            ProjectWS.ProjectDataSet.TaskCustomFieldsRow cfrow = project.TaskCustomFields.NewTaskCustomFieldsRow();
            cfrow.PROJ_UID = prjID;
            cfrow.TASK_UID = taskID;
            cfrow.MD_PROP_UID = new Guid(GetMD_PROP_UID(name));
            cfrow.DATE_VALUE = datevalue;
            cfrow.CUSTOM_FIELD_UID = Guid.NewGuid();
            project.TaskCustomFields.AddTaskCustomFieldsRow(cfrow);
        }

        public static void UpdateProjectCustomField(Guid cfUID, decimal value, ProjectWS.ProjectDataSet project)
        {
            ProjectWS.ProjectDataSet.ProjectCustomFieldsRow cfrow = project.ProjectCustomFields.FindByCUSTOM_FIELD_UID(cfUID);
            cfrow.NUM_VALUE = value;
        }

        public static void UpdateTaskCustomField(Guid cfUID, string value, ProjectWS.ProjectDataSet project)
        {
            ProjectWS.ProjectDataSet.TaskCustomFieldsRow cfrow = project.TaskCustomFields.FindByCUSTOM_FIELD_UID(cfUID);
            cfrow.TEXT_VALUE = value;
        }

        public static void UpdateTaskCustomField(Guid cfUID, decimal number, ProjectWS.ProjectDataSet project)
        {
            ProjectWS.ProjectDataSet.TaskCustomFieldsRow cfrow = project.TaskCustomFields.FindByCUSTOM_FIELD_UID(cfUID);
            cfrow.NUM_VALUE = number;
        }

        public static void UpdateTaskCustomField(Guid cfUID, Guid ltcode, ProjectWS.ProjectDataSet project)
        {
            ProjectWS.ProjectDataSet.TaskCustomFieldsRow cfrow = project.TaskCustomFields.FindByCUSTOM_FIELD_UID(cfUID);
            cfrow.CODE_VALUE = ltcode;
        }

        public static Guid FindTaskCustomFieldIDByName(Guid projID, Guid taskID, string cfname)
        {
            ProjectWS.ProjectDataSet projectDS = Program.project.ReadProject(projID, ProjectWS.DataStoreEnum.WorkingStore);
            return FindTaskCustomFieldIDByName(projID, taskID, cfname, projectDS);
        }

        public static Guid FindTaskCustomFieldIDByName(Guid projID, Guid taskID, string cfname, ProjectWS.ProjectDataSet dataset)
        {
            if (dataset != null)
            {
                dataset.TaskCustomFields.DefaultView.RowFilter = "PROJ_UID = '" + projID.ToString() + "' AND TASK_UID = '" + taskID.ToString() + "' AND MD_PROP_UID = '" + PSICustomFields.GetMD_PROP_UID(cfname) + "'";
                if (dataset.TaskCustomFields.DefaultView.Count > 0)
                    return new Guid(dataset.TaskCustomFields.DefaultView[0]["CUSTOM_FIELD_UID"].ToString());
                else
                    return Guid.Empty;
            }
            else
                return Guid.Empty;
        }

        //assignments
        public static void AddAssignmentCustomField(Guid prjID, Guid assnID, string name, string value, ProjectWS.ProjectDataSet project)
        {
            ProjectWS.ProjectDataSet.AssignmentCustomFieldsRow cfrow = project.AssignmentCustomFields.NewAssignmentCustomFieldsRow();
            cfrow.PROJ_UID = prjID;
            cfrow.ASSN_UID = assnID;
            cfrow.MD_PROP_UID = new Guid(GetMD_PROP_UID_SECONDARY(name));
            cfrow.TEXT_VALUE = value;
            cfrow.CUSTOM_FIELD_UID = Guid.NewGuid();
            project.AssignmentCustomFields.AddAssignmentCustomFieldsRow(cfrow);
        }

        public static void AddAssignmentCustomField(Guid prjID, Guid assnID, string name, decimal number, ProjectWS.ProjectDataSet project)
        {
            ProjectWS.ProjectDataSet.AssignmentCustomFieldsRow cfrow = project.AssignmentCustomFields.NewAssignmentCustomFieldsRow();
            cfrow.PROJ_UID = prjID;
            cfrow.ASSN_UID = assnID;
            cfrow.MD_PROP_UID = new Guid(GetMD_PROP_UID_SECONDARY(name));
            cfrow.NUM_VALUE = number;
            cfrow.CUSTOM_FIELD_UID = Guid.NewGuid();
            project.AssignmentCustomFields.AddAssignmentCustomFieldsRow(cfrow);
        }

        public static void AddAssignmentCustomFieldCOST(Guid prjID, Guid assnID, string name, decimal number, ProjectWS.ProjectDataSet project)
        {
            ProjectWS.ProjectDataSet.AssignmentCustomFieldsRow cfrow = project.AssignmentCustomFields.NewAssignmentCustomFieldsRow();
            cfrow.PROJ_UID = prjID;
            cfrow.ASSN_UID = assnID;
            cfrow.MD_PROP_UID = new Guid(GetMD_PROP_UID_SECONDARY(name));
            cfrow.NUM_VALUE = number;
            cfrow.CUSTOM_FIELD_UID = Guid.NewGuid();
            project.AssignmentCustomFields.AddAssignmentCustomFieldsRow(cfrow);
        }

        public static void UpdateAssignmentCustomField(Guid cfUID, string value, ProjectWS.ProjectDataSet project)
        {
            ProjectWS.ProjectDataSet.AssignmentCustomFieldsRow cfrow = project.AssignmentCustomFields.FindByCUSTOM_FIELD_UID(cfUID);
            cfrow.TEXT_VALUE = value;
        }

        public static void UpdateAssignmentCustomField(Guid cfUID, decimal number, ProjectWS.ProjectDataSet project)
        {
            ProjectWS.ProjectDataSet.AssignmentCustomFieldsRow cfrow = project.AssignmentCustomFields.FindByCUSTOM_FIELD_UID(cfUID);
            cfrow.NUM_VALUE = number;
        }

        public static void UpdateAssignmentCustomField(Guid cfUID, DateTime date, ProjectWS.ProjectDataSet project)
        {
            ProjectWS.ProjectDataSet.AssignmentCustomFieldsRow cfrow = project.AssignmentCustomFields.FindByCUSTOM_FIELD_UID(cfUID);
            cfrow.DATE_VALUE = date;
        }

        public static Guid FindAssignmentCustomFieldIDByName(Guid projID, Guid assnID, string cfname)
        {
            ProjectWS.ProjectDataSet projectDS = Program.project.ReadProject(projID, ProjectWS.DataStoreEnum.WorkingStore);
            return FindAssignmentCustomFieldIDByName(projID, assnID, cfname, projectDS);
        }

        public static Guid FindAssignmentCustomFieldIDByName(Guid projID, Guid assnID, string cfname, ProjectWS.ProjectDataSet dataset)
        {
            if (dataset != null)
            {
                dataset.AssignmentCustomFields.DefaultView.RowFilter = "PROJ_UID = '" + projID.ToString() + "' AND ASSN_UID = '" + assnID.ToString() + "' AND MD_PROP_UID = '" + PSICustomFields.GetMD_PROP_UID_SECONDARY(cfname) + "'";
                if (dataset.AssignmentCustomFields.DefaultView.Count > 0)
                    return new Guid(dataset.AssignmentCustomFields.DefaultView[0]["CUSTOM_FIELD_UID"].ToString());
                else
                    return Guid.Empty;
            }
            else
                return Guid.Empty;
        }
    }
}
