using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Data;
using PSLibrary = Microsoft.Office.Project.Server.Library;

namespace MicrosoftFrance.MCS.EPM2007
{
    class PSIMethods
    {
        public static Guid GetCalendar(string calendarname)
        {
            CalendarWS.CalendarDataSet calendarDataSet = new CalendarWS.CalendarDataSet();
            PSLibrary.Filter filter = new PSLibrary.Filter();
            PSLibrary.Filter.FieldBase field = new PSLibrary.Filter.Field("Calendars", "CAL_NAME");
            filter.Fields.Add(field);
            field = new PSLibrary.Filter.Field("Calendars", "CAL_UID");
            filter.Fields.Add(field);
            filter.FilterTableName = "Calendars";
            calendarDataSet = Program.calendar.ReadCalendars(filter.GetXml(), false);

            foreach (CalendarWS.CalendarDataSet.CalendarsRow calendarRow in calendarDataSet.Calendars.Rows)
            {
                if (calendarRow.CAL_NAME.ToLower().Contains(calendarname.ToLower()))
                {
                    return calendarRow.CAL_UID;
                }
            }
            return Guid.Empty;
        }

        public static Guid CreateTask(Guid prjID, string taskname, int taskduration, DateTime taskStartDate, ProjectWS.ProjectDataSet dataset)
        {
            ProjectWS.ProjectDataSet.TaskRow taskRow = dataset.Task.NewTaskRow();
            taskRow.PROJ_UID = prjID;
            taskRow.TASK_UID = Guid.NewGuid();
            taskRow.TASK_NAME = taskname;
            taskRow.TASK_DUR = taskduration;
            taskRow.TASK_DUR_IS_EST = false;
            taskRow.TASK_TYPE = 1; //fixed duration
            taskRow.TASK_CONSTRAINT_TYPE = (short)PSLibrary.Task.ConstraintType.StartNoEarlierThan;
            taskRow.TASK_CONSTRAINT_DATE = taskStartDate;
            dataset.Task.AddTaskRow(taskRow);
            return taskRow.TASK_UID;
        }

        public static void UpdateTaskDuration(Guid taskID, Guid prjID, string taskname, int taskduration, DateTime taskStartDate, ProjectWS.ProjectDataSet dataset)
        {
            ProjectWS.ProjectDataSet.TaskRow taskRow = dataset.Task.FindByTASK_UIDPROJ_UID(taskID, prjID);
            taskRow.TASK_NAME = taskname;
            taskRow.TASK_DUR = taskduration;
            //taskRow.TASK_DUR_IS_EST = false;
            //taskRow.TASK_TYPE = 1; //fixed duration
            taskRow.TASK_CONSTRAINT_TYPE = (short)PSLibrary.Task.ConstraintType.StartNoEarlierThan;
            taskRow.TASK_CONSTRAINT_DATE = taskStartDate;
            /*taskRow.TASK_WORK = taskwork;*/
            //taskRow.
            //dataset.Task.LoadDataRow((taskRow);
        }

        public static void UpdateTaskWork(Guid taskID, Guid prjID, double taskwork, ProjectWS.ProjectDataSet dataset)
        {
            ProjectWS.ProjectDataSet.TaskRow taskRow = dataset.Task.FindByTASK_UIDPROJ_UID(taskID, prjID);
            taskRow.TASK_WORK = taskwork;
        }

        public static Guid CreateAssignment(Guid prjID, Guid taskID, Guid resID, string taskname, double work, ProjectWS.ProjectDataSet dataset)
        {
            ProjectWS.ProjectDataSet.AssignmentRow assnRow = dataset.Assignment.NewAssignmentRow();
            assnRow.ASSN_UID = Guid.NewGuid();
            assnRow.PROJ_UID = prjID;
            assnRow.RES_UID = resID;
            assnRow.TASK_UID = taskID;
            assnRow.TASK_NAME = taskname;
            assnRow.ASSN_WORK = work;
            assnRow.ASSN_UNITS = 100; // 10%
            dataset.Assignment.AddAssignmentRow(assnRow);
            return assnRow.ASSN_UID;
        }
        public static Guid CreateAssignmentSummary(Guid prjID, Guid taskID, Guid resID, double budgetwork, ProjectWS.ProjectDataSet dataset)
        {
            ProjectWS.ProjectDataSet.AssignmentRow assnRow = dataset.Assignment.NewAssignmentRow();
            assnRow.ASSN_UID = Guid.NewGuid();
            assnRow.PROJ_UID = prjID;
            assnRow.RES_UID = resID;
            assnRow.TASK_UID = taskID;
            assnRow.ASSN_BUDGET_WORK = budgetwork;
            dataset.Assignment.AddAssignmentRow(assnRow);
            return assnRow.ASSN_UID;
        }

        public static void UpdateAssignmentSummaryBudgetWork(Guid assnID, Guid prjID, double budgetwork, ProjectWS.ProjectDataSet dataset)
        {
            ProjectWS.ProjectDataSet.AssignmentRow assnRow = dataset.Assignment.FindByASSN_UIDPROJ_UID(assnID, prjID);
            assnRow.ASSN_BUDGET_WORK = budgetwork;
        }

        public static void UpdateAssignmentWork(Guid assnID, Guid prjID, Guid taskID, double work, ProjectWS.ProjectDataSet dataset)
        {
            ProjectWS.ProjectDataSet.AssignmentRow assnRow = dataset.Assignment.FindByASSN_UIDPROJ_UID(assnID, prjID);
            assnRow.ASSN_WORK = work;
        }

        public static void UpdateAssignmentActualWork(Guid assnID, Guid prjID, Guid taskID, double actwork, ProjectWS.ProjectDataSet dataset)
        {
            ProjectWS.ProjectDataSet.AssignmentRow assnRow = dataset.Assignment.FindByASSN_UIDPROJ_UID(assnID, prjID);
            assnRow.ASSN_ACT_WORK = actwork;
        }

        public static void UpdateAssignment(Guid assnID, Guid prjID, Guid taskID, double work, double actwork, ProjectWS.ProjectDataSet dataset)
        {
            ProjectWS.ProjectDataSet.AssignmentRow assnRow = dataset.Assignment.FindByASSN_UIDPROJ_UID(assnID, prjID);
            //assnRow.ASSN_WORK = work;
            assnRow.ASSN_ACT_WORK = actwork;
            //assnRow.ASSN_REM_WORK = work - actwork;
            //assnRow.ASSN_REG_WORK = work;
        }

        public static Guid GetEnterpriseResIDFromName(string resname, ResourceWS.ResourceDataSet resources)
        {
            Guid resID = new Guid();
            resources.Resources.DefaultView.RowFilter = "";
            resID = new Guid(resources.Resources.DefaultView[0]["RES_UID"].ToString());
            return resID;
        }

        public static Guid FindTaskIDByName(Guid prjID, string taskname, ProjectWS.ProjectDataSet projDS)
        {
            //ProjectWS.ProjectDataSet projects = Program.project.ReadProject(prjID, ProjectWS.DataStoreEnum.WorkingStore);
            if (projDS != null)
            {
                projDS.Task.DefaultView.RowFilter = "TASK_NAME = '" + taskname + "'";
                if (projDS.Task.DefaultView.Count < 1)
                    return Guid.Empty;
                else if (projDS.Task.DefaultView.Count > 1)
                    throw new Exception("2 tches nommes \"" + taskname + "\" existent dans le projet \"" + projDS.Project[0].PROJ_NAME + "\"");
                else
                    return new Guid(projDS.Task.DefaultView[0]["TASK_UID"].ToString());
            }
            else
                return Guid.Empty;
        }

        public static Guid FindTaskCustomFieldIDByName(Guid projID, Guid taskID, string cfname, ProjectWS.ProjectDataSet dataset)
        {
            if (dataset != null)
            {
                dataset.TaskCustomFields.DefaultView.RowFilter = "PROJ_UID = '" + projID.ToString() + "' AND TASK_UID = '" + taskID.ToString() + "' AND MD_PROP_UID = '" + PSICustomFields.GetMD_PROP_UID(cfname) + "'";
                return new Guid(dataset.TaskCustomFields.DefaultView[0]["CUSTOM_FIELD_UID"].ToString());
            }
            else
                return Guid.Empty;

        }
    }
}
